#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2017-04-15:
#
# Cisco SN2INFO API Framework
#
# https://developer.cisco.com/docs/support-apis/#serial-number-to-information
#
# 2021-07-23: rewrite for python 3.8
#
import json

from cisco_live_cycle_utils import (
    configure_logger,
    log_message,
    expand_path,
    get_ids_from_dir,
    remove_ids_from_list,
    refresh_ids_from_dir,
    remove_ids_from_dir,
    sleep_random,
)
from ciscoapi import (
    AccessToken,
    Settings,
    get_coverage_summary_by_serials,
)


def sn2info_split_covered(sn2info_records):
    sn2info_covered = []
    sn2info_notcovered = []

    for response in sn2info_records:
        sn2inforecord = response.get('serial_numbers')
#        PaginationResponseRecord = response.get('PaginationResponseRecord')
#        PageIndex = PaginationResponseRecord.get('PageIndex')
#        LastIndex = PaginationResponseRecord.get('LastIndex')
#        PageRecords = PaginationResponseRecord.get('PageRecords')
#        TotalRecords = PaginationResponseRecord.get('TotalRecords')
        if sn2inforecord is not None:
            for serial in sn2inforecord:
                if serial.get('is_covered') == 'YES':
                    sn2info_covered.append(serial)
                    log_message(f'SN2INFO covered  : {serial.get("sr_no")}')
                else:
                    sn2info_notcovered.append(serial)
                    log_message(f'SN2INFO not covered : {serial.get("sr_no")}')
    return {'sn2info_covered': sn2info_covered, 'sn2info_notcovered': sn2info_notcovered}


def sn2info_save_serials(sn2infos, path):
    saved = []

    for sn2info in sn2infos:
        serial = str(sn2info.get('sr_no'))
        if serial != '':
            with open(path + serial, 'w') as f:
                json.dump(sn2info, f)
            saved.append(serial)

    return saved


def main():
    settings = Settings()
    access_token = AccessToken(settings.client_id, settings.client_secret, settings.proxies)
    configure_logger(log_level=settings.log_level)

    sn2info_dir = settings.base_path + '/sn2info'
    path_found = expand_path(sn2info_dir + '/found/')
    path_not_found = expand_path(sn2info_dir + '/not_found/')
    path_request = expand_path(sn2info_dir + '/request/')

    # create list of serial numbers to request SN2INFO status for
    sn2info = get_ids_from_dir(path_request)
    log_message(f'sn2info requests : {sn2info}')
    # remove covered serials from list
    sn2info = remove_ids_from_list(sn2info, path_found)
    log_message(f'sn2info requests : {sn2info}')
    # remove not covered serials from list
    sn2info = remove_ids_from_list(sn2info, path_not_found)
    log_message(f'sn2info requests : {sn2info}')

    # refresh sn2info serials after 31 days by default
    sn2info = refresh_ids_from_dir(path_not_found, settings.sn2info_refresh_not_covered, sn2info, True)
    log_message(f'sn2info requests : {sn2info}')
    sn2info = refresh_ids_from_dir(path_found, settings.sn2info_refresh_covered, sn2info, False)
    log_message(f'sn2info requests : {sn2info}')

    if sn2info is []:
        log_message('all list are empty. Do nothing.')
        return

    # wait random time after startup
    if settings.wait_after_start:
        sleep_random(settings.max_wait_time)

    if sn2info is not []:
        sn2info_records = get_coverage_summary_by_serials(
            serials=sn2info,
            access_token=access_token,
            settings=settings
        )
        log_message(f'sn2info response: {sn2info_records}')
        sn2info = sn2info_split_covered(sn2info_records)
        serials = sn2info_save_serials(sn2info.get('sn2info_covered'), path_found)
        remove_ids_from_dir(serials, path_request)
        serials = sn2info_save_serials(sn2info.get('sn2info_notcovered'), path_not_found)
        remove_ids_from_dir(serials, path_request)
        # delete serials from covered were the status changed to uncovered
        remove_ids_from_dir(serials, path_found)


main()
